/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 2009 Toporov Konstantin. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 3 only ("GPL")  (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.gnu.org/copyleft/gpl.html  See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * You should include file containing license in each project.
 * If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * If you wish your version of this file to be governed by
 * the GPL Version 3, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [GPL Version 3] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under the GPL Version 3 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 3 code and therefore, elected the GPL
 * Version 3 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 *
 * Contributor(s):
 *
 * Portions Copyrighted 2009 Toporov Konstantin.
 */
package lookout;

/**
 *
 * @author Toporov Konstantin <www.diacalc.org>
 */
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import lookout.settings.ProgramSettings;
import maths.*;
import java.io.*;
import java.net.*;
import java.util.*;
import products.*;
import manager.*;
import java.beans.*;
import javax.swing.event.*;


public class InetBackUpDialog extends JDialog implements ActionListener,
        PropertyChangeListener{
    private ProgramSettings settings = ProgramSettings.getInstance();
    private MainFrame owner;
    private JTextArea log;
    private JScrollPane scr;
    private JProgressBar prBar;
    private JRadioButton rbIm1;
    private JRadioButton rbIm2;
    private JRadioButton rbIm3;
    private InetUser inetData;
    private JCheckBox chImMenu;
    private JCheckBox chExMenu;
    private JCheckBox chExProducts;
    private JCheckBox chImProducts;
    private JCheckBox chImDiary;
    private JButton btnStExport;
    private JButton btnStImport;
    private JButton btnSync;
    private JButton btnChPass;
    private JRadioButton rbServerMajor;
    private JRadioButton rbLocalMajor;

    private JTextField fldLogin;
    private JPasswordField fldPass;
    private JTextField fldServer;
    private boolean sync = false;
    private InetUserManager inetMgr = new InetUserManager();

    private final static String SCRIPT = "server.php";

    private final static String BEGIN_EXPORT = "begin export";
    private final static String BEGIN_IMPORT = "begin import";
    private final static String CANCEL_TASKS = "cancel background tastks";
    private final static String BEGIN_SYNC = "begin sycronisation";
    private final static String CHANGE_PASS = "change password";
    

    private ExportTask taskExport = null;
    private ImportTask taskImport = null;
    private ImportBaseTask taskImportBase = null;
    private ChangePassTask taskChPass = null;

 
    private boolean cancel_was = false;
    private boolean hasCheckedVer = false;
    
    public InetBackUpDialog(MainFrame owner){
        super(owner);
        setTitle("Импорт и экспорт в интеренет базу данных");
        this.owner = owner;
        inetData = inetMgr.getUser();
        JTabbedPane tabPane = new JTabbedPane();
        tabPane.add("Синхронизация",createSyncPane());
        tabPane.add("Экспорт",createExportPane());
        tabPane.add("Импорт",createImportPane());
        JPanel pane = new JPanel(new BorderLayout());
        pane.add(tabPane,BorderLayout.CENTER);
        this.add(pane);
        add(createTopPane(),BorderLayout.NORTH);
        add(createBottomPane(),BorderLayout.SOUTH);

        this.pack();
        this.setLocationRelativeTo(this.owner);
        this.setResizable(false);
        this.setDefaultCloseOperation(JDialog.DO_NOTHING_ON_CLOSE);
        this.addWindowListener(new WindowAdapter(){
               @Override
               public void windowClosing(WindowEvent e){
                   closeDialog();
               }
        });
    }
    private void closeDialog(){
        if (taskExport!=null && !taskExport.isDone()) taskExport.cancel(true);
        if (taskImport!=null && !taskImport.isDone()) taskImport.cancel(true);
        if (taskImportBase!=null && !taskImportBase.isDone()) taskImportBase.cancel(true);
        if (taskChPass!=null && !taskChPass.isDone()) taskChPass.cancel(true);

        setVisible(false);
        saveLogInfo();
    }
    private void enableElements(boolean enable){
        chImProducts.setEnabled(enable);
        rbIm1.setEnabled(enable && chImProducts.isSelected());
        rbIm2.setEnabled(enable && chImProducts.isSelected());
        rbIm3.setEnabled(enable && chImProducts.isSelected());
        chImMenu.setEnabled(enable);
        chImDiary.setEnabled(enable);
        chExMenu.setEnabled(enable);
        chExProducts.setEnabled(enable);
        btnChPass.setEnabled(enable);
        rbServerMajor.setEnabled(enable);
        rbLocalMajor.setEnabled(enable);
    }
    public void actionPerformed(ActionEvent ev){
        String cmd = ev.getActionCommand();
        if (BEGIN_EXPORT.equals(cmd)){
            cancelButtons();
            
            taskExport = new ExportTask(chExMenu.isSelected(),chExProducts.isSelected());
            taskExport.addPropertyChangeListener(this);
            taskExport.execute();
        }
        else if (BEGIN_IMPORT.equals(cmd)){
            cancelButtons();

            taskImport = new ImportTask(
                    chImMenu.isSelected(),
                    chImProducts.isSelected(),
                    chImDiary.isSelected()
                    /*rbIm1.isSelected()?ImportBaseTask.MODE_UPDATE:
                        (rbIm2.isSelected()?ImportBaseTask.MODE_ADD:ImportBaseTask.MODE_REPLACE)*/
                    );
            taskImport.addPropertyChangeListener(this);
            taskImport.execute();
        }
        else if (CANCEL_TASKS.equals(cmd)){
            cancel_was = true; //чего бы там дальше не было - не делаем
            log.append("Операция прервана пользователем.\n");

            prBar.setIndeterminate(false);
            prBar.setValue(0);
            restoreButtons();

            if (taskImport!=null && !taskImport.isDone()) taskImport.cancel(true);
            if (taskImportBase!=null && !taskImportBase.isDone()) taskImportBase.cancel(true);
            if (taskExport!=null && !taskExport.isDone()) taskExport.cancel(true);
            if (taskChPass!=null && !taskChPass.isDone()) taskChPass.cancel(true);
        }
        else if (BEGIN_SYNC.equals(cmd)){
            cancelButtons();
            log.append("Синхронизация начата...\n");
            sync = true;
            
            ///////Сначала импортируем в режиме объединения
            taskImport = new ImportTask(
                    false,
                    true,
                    false//ImportBaseTask.MODE_UPDATE
                    );

            taskImport.addPropertyChangeListener(this);
            taskImport.execute();
        }
        else if (CHANGE_PASS.equals(cmd)){
            //Меняем пароль
            changePassword();
        }
        
    }
    private void changePassword(){
        JLabel lbl1 = new JLabel(settings.getIn().StrSize("Введите новый пароль"));
        JPasswordField pass1 = new JPasswordField();
        JLabel lbl2 = new JLabel(settings.getIn().StrSize("Введите пароль повторно"));
        JPasswordField pass2 = new JPasswordField();
        JLabel lbl3 = new JLabel(settings.getIn().StrSize(
                "Пароль будет изменен, если в форме<br>" +
                "импорта и экспорта указаны верные<br>" +
                "действующие логин и пароль"));
        Object [] arr = {lbl1,pass1,lbl2,pass2,lbl3};
        String [] var = {settings.getIn().StrSize("Изменить"),
        settings.getIn().StrSize("Отмена")};
        if(JOptionPane.showOptionDialog(
                                        this,
                                        arr,
                                        "Ввод СК",
                                        JOptionPane.OK_CANCEL_OPTION,
                                        JOptionPane.PLAIN_MESSAGE,
                                        null,
                                        var,
                                        var[0])==JOptionPane.OK_OPTION){
            if (pass1.getPassword().length==pass2.getPassword().length ||
                    !new String(pass1.getPassword()).equals( new String(pass2.getPassword()) ) ){
                cancelButtons();
                log.append("Меняем пароль...\n");

                taskChPass = new ChangePassTask(new String(pass1.getPassword()));
                taskChPass.addPropertyChangeListener(this);
                taskChPass.execute();
            }
            else log.append("Введеные пароли не совпадают\n");
            //yes
        }
    }
    private  void cancelButtons(){
        enableElements(false);

            prBar.setIndeterminate(true);
            btnStExport.setActionCommand(CANCEL_TASKS);
            btnStExport.setText("Отменить");
            btnStImport.setActionCommand(CANCEL_TASKS);
            btnStImport.setText("Отменить");
            btnSync.setActionCommand(CANCEL_TASKS);
            btnSync.setText("Отменить");
    }
    public void propertyChange(PropertyChangeEvent evt) {
        if ("state".equals(evt.getPropertyName())) {
            if (evt.getSource()==taskImportBase && 
                    taskImportBase.getState()==SwingWorker.StateValue.DONE){
                if (cancel_was){
                    cancel_was = false;
                    restoreButtons();
                }else{
                    //Закончили добавлять или обновлять продукты в базе
                    log.append("Импорт продуктов в базу закончен\n");
                    if (sync){//теперь нужно запустить экспорт нет, сначала нужно запустить импорт в базу
                        taskExport = new ExportTask(false,true);
                        taskExport.addPropertyChangeListener(this);
                        taskExport.execute();
                    }else{//закончили нормально импорт
                        restoreButtons();
                        owner.refreshProducts();
                    }
                }
                taskImportBase = null;
                System.gc();
            }
            else if (evt.getSource()==taskImport && 
                    taskImport.getState()==SwingWorker.StateValue.DONE){
                if (cancel_was){
                    cancel_was = false;
                    restoreButtons();
                }
                //Закончили импорт, теперь проверяем нужно ли импортировать базу продуктов
                else if ( chImProducts.isSelected() || sync ){
                    /////////////////
                    Vector<ProdGroup> groups = taskImport.getGroups();
                    Vector<ProductInBase> prods = taskImport.getProds();
                    Vector<ComplexProduct> cmpl = taskImport.getCmpl();
                    if (groups!=null && prods!=null && cmpl!=null){
                        boolean goon = true;
                        int locprods=0;// = new ProductManager().getAllProducts().size();
                        int mode = rbIm1.isSelected()?ImportBaseTask.MODE_UPDATE:
                            (rbIm2.isSelected()?ImportBaseTask.MODE_ADD:ImportBaseTask.MODE_REPLACE);
                        if (mode==ImportBaseTask.MODE_REPLACE &&
                            prods.size()<(locprods=new ProductManager().getAllProducts().size())
                                && !sync){
                                //загружаемых продуктов меньше, чем в текущей базе
                                Object[] options = {settings.getIn().StrSize("Да"),
                                    settings.getIn().StrSize("Нет") };
                                goon = JOptionPane.showOptionDialog(InetBackUpDialog.this,
                                    settings.getIn().StrSize(
                                    "В загружаемой базе продкутов меньше, чем локальной базе<br>" +
                                    "Из локальной базы будет удалено "+locprods+" продуктов<br>"+
                                    "Будет импортировано "+prods.size()+" продуктов<br><br>" +
                                    "Продолжить?"),
                                    "Импорт",
                                    JOptionPane.YES_NO_OPTION,
                                    JOptionPane.WARNING_MESSAGE,
                                    null,
                                    options,
                                    options[1])==JOptionPane.YES_OPTION;
                        }
                        if (goon || sync){
                            log.append("Идет импорт продуктов в базу, подождите...\n");
                            int maj = ImportBaseTask.LOCAL;
                            if (sync && rbServerMajor.isSelected()) maj = ImportBaseTask.SERVER;

                            taskImportBase = new ImportBaseTask(groups,prods,cmpl,mode,maj);
                            taskImportBase.addPropertyChangeListener(InetBackUpDialog.this);
                            taskImportBase.execute();
                        }
                    }

                    /////////////////
                }//Значит закончили теперь нужно вернуть кнопки
                else{
                    if (chImDiary.isSelected()) owner.refreshUser();
                    restoreButtons();
                }
                taskImport = null;
                System.gc();
            }
            else if (evt.getSource()==taskExport &&
                    taskExport.getState()==SwingWorker.StateValue.DONE){// &&  !chImProducts.isSelected()
                if (cancel_was){
                    cancel_was = false;
                    restoreButtons();
                }
                else{
                    if (sync){
                        log.append("Синхронизация завершена.\n");
                        owner.refreshProducts();
                        sync = false;
                    }
                    restoreButtons();
                }
                taskExport = null;
                System.gc();
            }else if (evt.getSource()==taskChPass &&
                    taskChPass.getState()==SwingWorker.StateValue.DONE){// &&  !chImProducts.isSelected()
                if (cancel_was){
                    cancel_was = false;
                }
                restoreButtons();
                taskChPass = null;
                System.gc();
            }
            
        }else if ("progress".equals(evt.getPropertyName())) {
            int progress = (Integer) evt.getNewValue();
            if (evt.getSource()==taskExport){
                prBar.setIndeterminate(taskExport.getIntermediate());
                prBar.setValue(progress);
            }
            else{
                prBar.setValue(progress);
            }
        }
    }
    private void restoreButtons(){
        btnStExport.setActionCommand(BEGIN_EXPORT);
        btnStExport.setText("Начать");

        btnStImport.setActionCommand(BEGIN_IMPORT);
        btnStImport.setText("Начать");

        btnSync.setActionCommand(BEGIN_SYNC);
        btnSync.setText("Синхронизация");

        prBar.setIndeterminate(false);
        prBar.setValue(0);
        enableElements(true);
    }
    
    private String addKeyValue(String query, String key, String value){
        String res;
        if (query==null){
            res = "";
        }
        else res = query;
        if (res.length()>0){
            res += "&";
        }
        try{
            res += key + "=" + URLEncoder.encode (value,"UTF8");
        }
        catch (UnsupportedEncodingException ex){
            res += key + "=" + value;
        }
        return res;
    }
    private String getFirstTagged(String output, String tag){
        String tagfullst = "<"+tag+">";
        String tagfullend = "</"+tag+">";
        int st = output.indexOf(tagfullst);
        int end = output.indexOf( tagfullend, st+tagfullst.length());
        return output.substring(st+tagfullst.length(), end);
    }
    private Vector<DiaryUnit> getDiaryComments(String st){
        Vector<DiaryUnit> arr = new Vector<DiaryUnit>();
        String [] lines = st.trim().split("\n");
        for(int i=0;i<lines.length;i++){
            String [] param = lines[i].split(" ");
            if (param.length>1){
                long time = new Long(param[param.length-1]);
                String comm ="";
                for(int j=0;j<(param.length-1);j++){
                    comm += param[j] + " ";
                }
                comm = comm.trim();
                arr.add(new DiaryUnit(-1,time,comm,owner.getUser().getId()));
            }
        }
        return arr;
    }
    private Vector<DiaryUnit> getDiarySugars(String st){
        Vector<DiaryUnit> arr = new Vector<DiaryUnit>();
        String [] lines = st.trim().split("\n");
        for(int i=0;i<lines.length;i++){
            String [] param = lines[i].split(" ");
            if (param.length>2){
                float sh = new Float(param[param.length-1]);
                long time = new Long(param[param.length-2]);
                String comm ="";
                for(int j=0;j<(param.length-2);j++){
                    comm += param[j] + " ";
                }
                comm = comm.trim();
                arr.add(new DiaryUnit(-1,time,comm,sh,owner.getUser().getId()));
            }
        }
        return arr;
    }
    private Vector<DiaryUnit> getDiaryMenus(String st){
        Vector<DiaryUnit> arr = new Vector<DiaryUnit>();
        String [] lines = st.trim().split("\n");
        for(int i=0;i<lines.length;i++){
            String [] param = lines[i].split(" ");
            if (param.length>13){//14
                float w = new Float(param[param.length-1]);
                int gi = new Integer(param[param.length-2]);
                float c = new Float(param[param.length-3]);
                float f = new Float(param[param.length-4]);
                float p = new Float(param[param.length-5]);
                float d = new Float(param[param.length-6]);
                float k3 = new Float(param[param.length-7]);
                float k2 = new Float(param[param.length-8]);
                float k1 = new Float(param[param.length-9]);
                float sh2 = new Float(param[param.length-10]);
                float sh1 = new Float(param[param.length-11]);
                long time = new Long(param[param.length-12]);
                int id = new Integer(param[param.length-13]);
                String comm ="";
                for(int j=0;j<(param.length-13);j++){
                    comm += param[j] + " ";
                }
                comm = comm.trim();
                arr.add(new DiaryUnit(
                        id,
                        time,
                        comm,
                        sh1,sh2,
                        new Factors(k1 * owner.getUser().getFactors().getBEValue() / 10f
                        ,k2,k3,owner.getUser().getFactors().getBEValue()),
                        d,
                        owner.getUser().getId(),
                        new ProductW("",p,f,c,gi,w)
                        ));
            }
        }
        return arr;
    }
    private Vector [] getDiaryRecords(String st){ //[menu-0,snack-1]
        Vector<ComplexProduct> menu = new Vector<ComplexProduct>();
        Vector<ComplexProduct> snack = new Vector<ComplexProduct>();
        String [] lines = st.trim().split("\n");
        for(int i=0;i<lines.length;i++){
            String [] param = lines[i].split(" ");
            if (param.length>7){
                int o = new Integer(param[param.length-1]);
                boolean isSnack = new Integer(param[param.length-2]).intValue() == 1;
                float w = new Float(param[param.length-3]);
                int gi = new Integer(param[param.length-4]);
                float c = new Float(param[param.length-5]);
                float f = new Float(param[param.length-6]);
                float p = new Float(param[param.length-7]);
                String name ="";
                for(int j=0;j<(param.length-7);j++){
                    name += param[j] + " ";
                }
                name = name.trim();
                
                if (isSnack){
                    snack.add(new ComplexProduct(
                            name,
                            p,f,c,gi,w,
                            -1,o
                            ));
                }else{
                    menu.add(new ComplexProduct(
                            name,
                            p,f,c,gi,w,
                            -1,o
                            ));
                }
                
            }
        }
        return new Vector[]{menu,snack};
    }
    private Vector<ProductInMenu> getMenu(String st,int type){
        Vector<ProductInMenu> arr = new Vector<ProductInMenu>();
        String [] lines = st.trim().split("\n");
        for(int i=0;i<lines.length;i++){
            String [] param = lines[i].split(" ");
            if (param.length>6){
                int typeP = new Integer(param[param.length-1]);
                if (type==typeP){
                    float w = new Float(param[param.length-2]);
                    int gi = new Integer(param[param.length-3]);
                    float c = new Float(param[param.length-4]);
                    float f = new Float(param[param.length-5]);
                    float p = new Float(param[param.length-6]);
                    String name ="";
                    for(int j=0;j<(param.length-6);j++){
                        name += param[j] + " ";
                    }
                    name = name.trim();
                    arr.add(new ProductInMenu(name,p,f,c,gi,w,0));
                }
            }
        }
        return arr;
    }
    private Vector<ProdGroup> getGroup(String st){
        Vector<ProdGroup> arr = new Vector<ProdGroup>();
        String [] lines = st.trim().split("\n");
        for(int i=0;i<lines.length;i++){
            if (lines[i].trim().length()>0){
                String [] param = lines[i].split(" ");
                if (param.length>1){
                    int id = new Integer(param[param.length-1]);
                
                    String name ="";
                    for(int j=0;j<(param.length-1);j++){
                       name += param[j] + " ";
                    }
                    name = name.trim();
                    arr.add(new ProdGroup(id,name,0));
            }
            }
        }
        return arr;
    }
    private Vector<ProductInBase> getProduct(String st){
        Vector<ProductInBase> arr = new Vector<ProductInBase>();
        String [] lines = st.trim().split("\n");
        for(int i=0;i<lines.length;i++){
          if (lines[i].trim().length()>0){
            String [] param = lines[i].split(" ");
            if (param.length>9){
                int own = new Integer(param[param.length-1]);
                int id = new Integer(param[param.length-2]);
                int usage = new Integer(param[param.length-3]);
                boolean cmpl = new Integer(param[param.length-4])==1;
                float w = new Float(param[param.length-5]);
                int gi = new Integer(param[param.length-6]);
                float c = new Float(param[param.length-7]);
                float f = new Float(param[param.length-8]);
                float p = new Float(param[param.length-9]);
                String name ="";
                for(int j=0;j<(param.length-9);j++){
                    name += param[j] + " ";
                }
                name = name.trim();
                arr.add(new ProductInBase(name,p,f,c,gi,w,id,cmpl,own,usage));
            }
          }
        }
        return arr;
    }
    private Vector<ComplexProduct> getComplex(String st){
        Vector<ComplexProduct> arr = new Vector<ComplexProduct>();
        String [] lines = st.trim().split("\n");
        for(int i=0;i<lines.length;i++){
          if (lines[i].trim().length()>0){
            String [] param = lines[i].split(" ");
            if (param.length>6){
                int own = new Integer(param[param.length-1]);
                float w = new Float(param[param.length-2]);
                int gi = new Integer(param[param.length-3]);
                float c = new Float(param[param.length-4]);
                float f = new Float(param[param.length-5]);
                float p = new Float(param[param.length-6]);
                String name ="";
                for(int j=0;j<(param.length-6);j++){
                    name += param[j] + " ";
                }
                name = name.trim();
                arr.add(new ComplexProduct(name,p,f,c,gi,w,0,own));
            }
          }
        }
        return arr;
    }
    private void saveLogInfo(){
        inetData.setLogin(fldLogin.getText());
        inetData.setPass(new String(fldPass.getPassword()));
        inetData.setServer(DoingPOST.checkServerPath(fldServer.getText()));
        inetMgr.setUser(inetData);
    }

    class ChangePassTask extends SwingWorker<Void, Void> {
        private String newpass;
        public ChangePassTask(String newpass){
            super();
            this.newpass = newpass;
        }
        @Override
        public Void doInBackground() {
            if (isCancelled()) return null;
            
            String query = addKeyValue("","login",inetData.getLogin());
            query = addKeyValue(query,"pass",inetData.getPass());
            query = addKeyValue(query,"action","change pass");
            query = addKeyValue(query,"newpass",newpass);
            query = addKeyValue(query,"ok","ok");
            
            DoingPOST post = new DoingPOST(inetData.getServer(),query);
            
            if (!post.isError()){
                log.append("Пароль успешно изменен\n" +
                        "Информация о новом пароле выслана на контактный email\n" +
                        "Новый пароль внесен в поле ввода пароля\n");
                fldPass.setText(newpass);
                inetData.setPass(newpass);
                saveLogInfo();
            }
            else log.append(post.getErrorMessage()+"\n");
            //тут надо проверить версию
            if (!hasCheckedVer){
                String r = checkVersion();
                if (r.length()>0) log.append("Доступно обновление программы\n"+r+"\n");
            }
            return null;
        }
        @Override
        public void done() {
            Toolkit.getDefaultToolkit().beep();
        }
        
    }
    class ImportTask extends SwingWorker<Void, Void> {
        private boolean inter = true;
        private Vector<ProdGroup> groups = null;
        private Vector<ProductInBase> prods = null;
        private Vector<ComplexProduct> cmpls = null;
        
        private boolean domenu;
        private boolean doproducts;
        private boolean dodiary;
        int maj;
       
        public ImportTask(boolean menu,boolean prod,boolean diary){
            super();
            domenu = menu;
            doproducts = prod;
            dodiary = diary;
        }
        public boolean getIntermediate(){
            return inter;
        }
        @Override
        public Void doInBackground() {
            saveLogInfo();

        //вначале импортируем меню
        if (domenu  && !isCancelled()){
            log.append("Импорт меню начат, подождите...\n");
            String query = addKeyValue("","login",inetData.getLogin());
            query = addKeyValue(query,"pass",inetData.getPass());
            query = addKeyValue(query,"action","get menu");
            query = addKeyValue(query,"ok","ok");
            
            DoingPOST post = new DoingPOST(inetData.getServer(),query);
                        
            if (!post.isError() && !isCancelled()){
                log.append("Меню скачано успешно\n");
                String menu = getFirstTagged(post.getAnswer(),"menu");
                String coefs = getFirstTagged(post.getAnswer(),"coefs");
                Vector<ProductInMenu> mn = getMenu(menu,0);
                Vector<ProductInMenu> sn = getMenu(menu,1);

                User us = extractCoefs(coefs);
                //Теперь открываем менеджеры, суем туда продукты, меняем коэф-ты
                //у юзера и сообщаем главному окну, что нужно бы обновить информацию
                if (us!=null && !isCancelled()){
                    MenuManager menuMgr =  new MenuManager(owner.getUser().getId(),
                         MenuManager.MENU_TABLE);

                    MenuManager snackMgr =  new MenuManager(owner.getUser().getId(),
                         MenuManager.SNACK_TABLE);
                    UsersManager usrsMgr = new UsersManager();
                    usrsMgr.updateFactors(us);
                    menuMgr.flush();
                    for(int i=0;(i<mn.size() && !isCancelled());i++){
                        menuMgr.addProd(mn.get(i));
                    }
                    snackMgr.flush();
                    for(int i=0;(i<sn.size() && !isCancelled());i++){
                        snackMgr.addProd(sn.get(i));
                    }
                    
                    owner.userChanged();
                    log.append("Меню импортировано успешно\n");
                }
            }
            else{
                if (!isCancelled()) log.append(post.getErrorMessage()+"\n");
                cancel_was = true;
            }
        }
        if (doproducts && !isCancelled() && !cancel_was){
            log.append("Импорт продуктов начат, подождите...\n");
            String query = addKeyValue("","login",inetData.getLogin());
            query = addKeyValue(query,"pass",inetData.getPass());
            query = addKeyValue(query,"action","get products");
            query = addKeyValue(query,"ok","ok");
            
            DoingPOST post = new DoingPOST(inetData.getServer(),query);
            
            
            if (!post.isError() && !isCancelled()){
                log.append("Продукты скачаны успешно\n");

                groups = getGroup(getFirstTagged(post.getAnswer(),"groups"));
                prods =getProduct(getFirstTagged(post.getAnswer(),"prods"));
                cmpls =getComplex(getFirstTagged(post.getAnswer(),"complex"));
            }
            else {
                if (!isCancelled()) log.append(post.getErrorMessage()+"\n");
                cancel_was = true;
            }
        }
        if (dodiary &&  !isCancelled()  && !cancel_was){
            log.append("Импорт записей дневника начат, подождите...\n");
            String query = addKeyValue("","login",inetData.getLogin());
            query = addKeyValue(query,"pass",inetData.getPass());
            query = addKeyValue(query,"action","get diary with snacks");
            query = addKeyValue(query,"ok","ok");
            
            DoingPOST post = new DoingPOST(inetData.getServer(),query);

            /*try{
                res = sendRequest(query);
            }
            catch(Exception ex){
                ex.printStackTrace();
            }*/

            if (!post.isError() && !isCancelled()){
                //импортируем
                log.append("Дневник скачан успешно...\n");
                
                Vector<DiaryUnit> duS = getDiarySugars(getFirstTagged(post.getAnswer(),"diarysugars"));
                Vector<DiaryUnit> duC = getDiaryComments(getFirstTagged(post.getAnswer(),"diarycomments"));
                Vector<DiaryUnit> duM = getDiaryMenus(getFirstTagged(post.getAnswer(),"diarymenus"));
                
                Vector [] duR = getDiaryRecords(getFirstTagged(post.getAnswer(),"diaryrecords")); //[menu,snack]

                DiaryManager dmgr = new DiaryManager(owner.getUser());
                for(int i=0;i<duS.size() && !isCancelled();i++){
                    dmgr.addUnit(duS.get(i));
                }
                for(int i=0;i<duC.size() && !isCancelled();i++){
                    dmgr.addComment(duC.get(i));
                }
                
                for(int i=0;i<duM.size() && !isCancelled();i++){
                    Collection<ProductInMenu>  [] menu = new Collection[] {new ArrayList(),new ArrayList()};
                    
                    Collection<ProductInMenu> snack = new ArrayList();
                    for(int m=0;m<2;m++){
                        for(int j=0;j<duR[m].size() && !isCancelled();j++){//Проходим по меню и перекусам
                            if ( ((ComplexProduct)duR[m].get(j)).getOwner()==duM.get(i).getId()){
                                menu[m].add(new ProductInMenu(
                                        ((ComplexProduct)duR[m].get(j)).getName(),
                                        ((ComplexProduct)duR[m].get(j)).getProt(),
                                        ((ComplexProduct)duR[m].get(j)).getFat(),
                                        ((ComplexProduct)duR[m].get(j)).getCarb(),
                                        ((ComplexProduct)duR[m].get(j)).getGi(),
                                        ((ComplexProduct)duR[m].get(j)).getWeight(),
                                        -1
                                        ));
                            }
                        }
                    }
                    
                    dmgr.addMenu(duM.get(i), menu[0], menu[1]);
                }
                log.append("Записи добавлены в локальную базу\nОчищаем записи на сервере...\n");
                query = addKeyValue("","login",inetData.getLogin());
                query = addKeyValue(query,"pass",inetData.getPass());
                query = addKeyValue(query,"action","clean diary");
                query = addKeyValue(query,"ok","ok");
                
                DoingPOST postInt = new DoingPOST(inetData.getServer(),query);

                if (!postInt.isError()){
                    log.append("Успешно сделано\n");
                }
                else{
                    if (!isCancelled()) log.append(postInt.getErrorMessage()+"\n");
                    cancel_was = true;
                }
            }
            else {
                if (!isCancelled()) log.append(post.getErrorMessage()+"\n");
                cancel_was = true;
            }
        }
            //тут надо проверить версию
            if (!hasCheckedVer){
                String r = checkVersion();
                if (r.length()>0) log.append("Доступно обновление программы\n"+r+"\n");
            }
            return null;
        }
        public Vector<ProdGroup> getGroups(){
            return groups;
        }
        public Vector<ProductInBase> getProds(){
            return prods;
        }
        public Vector<ComplexProduct> getCmpl(){
            return cmpls;
        }

        @Override
        public void done() {
            Toolkit.getDefaultToolkit().beep();
        }
    }
    private String checkVersion(){
        hasCheckedVer = true;
        String query = addKeyValue("","login",inetData.getLogin());
        query = addKeyValue(query,"pass",inetData.getPass());
        query = addKeyValue(query,"action","check version");
        query = addKeyValue(query,"pName","DCJ");
        query = addKeyValue(query,"pVer",""+MainFrame.PROGRAM_VER_INT);
        query = addKeyValue(query,"ok","ok");

        DoingPOST post = new DoingPOST(inetData.getServer(),query);
        
        if (!post.isError()){//Нормальный ответ
            return post.getAnswer().replace("<ok>", "");
        }else{
            return "Произошла ошибка при проверке версии\n"+post.getErrorMessage();
        }
    }

    class ExportTask extends SwingWorker<Void, Void> {
        
        private boolean inter = true;
        private boolean domenu;
        private boolean doproducts;

        public ExportTask(boolean m,boolean p){
            domenu = m;
            doproducts = p;
        }

        public boolean getIntermediate(){
            return inter;
        }
        
        @Override
        public Void doInBackground() {
            saveLogInfo();
        if (doproducts||domenu) log.append("Экспорт начат, подождите...\n");
        if (domenu && !isCancelled()){
            MenuManager menuMgr =  new MenuManager(owner.getUser().getId(),
                     MenuManager.MENU_TABLE);
             Vector menu = new Vector(menuMgr.getMenu());
             MenuManager snackMgr =  new MenuManager(owner.getUser().getId(),
                     MenuManager.SNACK_TABLE);
             Vector snack = new Vector(snackMgr.getMenu());
             if ((menu.size()+snack.size())>0 && !isCancelled()){
                String query = addKeyValue("","login",inetData.getLogin());
                query = addKeyValue(query,"pass",inetData.getPass());
                query = addKeyValue(query,"action","send menu");
                User us = owner.getUser();
                query = addKeyValue(query,"k1",""+us.getFactors().getK1Value()*10/us.getFactors().getBEValue() );
                query = addKeyValue(query,"k2",""+us.getFactors().getK2());
                query = addKeyValue(query,"k3",""+us.getFactors().getK3() );
                query = addKeyValue(query,"sh1",""+us.getSh1().getValue());
                query = addKeyValue(query,"sh2",""+us.getSh2().getValue());
                for(int i=0;(i<menu.size() && !isCancelled());i++){
                    ProductInMenu pr = (ProductInMenu)menu.get(i);
                    query = addKeyValue(query,"names[]", pr.getName());
                    query = addKeyValue(query,"prots[]", ""+pr.getProt());
                    query = addKeyValue(query,"fats[]", ""+pr.getFat());
                    query = addKeyValue(query,"carbs[]", ""+pr.getCarb());
                    query = addKeyValue(query,"gis[]", ""+pr.getGi());
                    query = addKeyValue(query,"weights[]", ""+pr.getWeight());
                    query = addKeyValue(query,"issnacks[]", "0");
                }
                for(int i=0;(i<snack.size() && !isCancelled());i++){
                    ProductInMenu pr = (ProductInMenu)snack.get(i);
                    query = addKeyValue(query,"names[]", pr.getName());
                    query = addKeyValue(query,"prots[]", ""+pr.getProt());
                    query = addKeyValue(query,"fats[]", ""+pr.getFat());
                    query = addKeyValue(query,"carbs[]", ""+pr.getCarb());
                    query = addKeyValue(query,"gis[]", ""+pr.getGi());
                    query = addKeyValue(query,"weights[]", ""+pr.getWeight());
                    query = addKeyValue(query,"issnacks[]", "1");
                }
                query = addKeyValue(query,"ok","ok");
                
                if (!isCancelled()){
                    DoingPOST post = new DoingPOST(inetData.getServer(),query);
                    
                    if (!post.isError()){
                        log.append("Меню экспортировано успешно\n");
                    }
                    else{
                        log.append(post.getErrorMessage()+"\n");
                    }
                }
             }
        }
        if (doproducts && !isCancelled()){
            
            GroupManager grMgr = new GroupManager();
            Vector<ProdGroup> groups = new Vector<ProdGroup>(grMgr.getGroups(GroupManager.ONLY_EXISTS_GROUPS));
            ProductManager prMgr = new ProductManager();
            Vector<ProductInBase> prods = new Vector<ProductInBase>(prMgr.getAllProducts());
            ComplexManager cmplMgr = new ComplexManager();
            Vector<ComplexProduct> cmpls = new Vector<ComplexProduct>();
            if (groups.size()>0 && !isCancelled()){
                String query = addKeyValue("","login",inetData.getLogin());
                query = addKeyValue(query,"pass",inetData.getPass());
                query = addKeyValue(query,"action","send products");
                String prquery;

                int elements = groups.size() + prods.size();
                int count = 0;
                this.inter = false;
                setProgress(0);

                for(int i=0;(i<groups.size() && !isCancelled());i++){
                    prquery = "";
                    prquery = addKeyValue(prquery,"grids[]",""+groups.get(i).getId());
                    prquery = addKeyValue(prquery,"grnames[]",groups.get(i).getName());
                    prquery = addKeyValue(prquery,"grindxs[]",""+groups.get(i).getSortInd());

                    query += "&"+prquery;
                    count++;
                    setProgress( (int)(100*(float)count/elements) );
                }
                
                for(int i=0;(i<prods.size() && !isCancelled());i++){
                    prquery = "";
                    prquery = addKeyValue(prquery,"prids[]",""+prods.get(i).getId());
                    prquery = addKeyValue(prquery,"prgrps[]",""+prods.get(i).getOwner());
                    prquery = addKeyValue(prquery,"prnames[]",prods.get(i).getName());
                    prquery = addKeyValue(prquery,"prprots[]",""+prods.get(i).getProt());
                    prquery = addKeyValue(prquery,"prfats[]",""+prods.get(i).getFat());
                    prquery = addKeyValue(prquery,"prcarbs[]",""+prods.get(i).getCarb());
                    prquery = addKeyValue(prquery,"prgis[]",""+prods.get(i).getGi());
                    prquery = addKeyValue(prquery,"prweights[]",""+prods.get(i).getWeight());
                    prquery = addKeyValue(prquery,"prcmpls[]",prods.get(i).isComplex()?"1":"0");
                    prquery = addKeyValue(prquery,"prusages[]",""+prods.get(i).getUsage());

                    query += "&"+prquery;
                    if (prods.get(i).isComplex()){
                        cmpls.addAll(cmplMgr.getComposition(prods.get(i).getId()));
                    }
                    count++;
                    setProgress( (int)(100*(float)count/elements) );
                }
                inter = true;
                setProgress(0);
                
                for(int i=0;(i<cmpls.size() && !isCancelled());i++){
                    prquery = "";

                    prquery = addKeyValue(prquery,"cmids[]",""+cmpls.get(i).getId());
                    prquery = addKeyValue(prquery,"cmprods[]",""+cmpls.get(i).getOwner());
                    prquery = addKeyValue(prquery,"cmnames[]",cmpls.get(i).getName());
                    prquery = addKeyValue(prquery,"cmprots[]",""+cmpls.get(i).getProt());
                    prquery = addKeyValue(prquery,"cmfats[]",""+cmpls.get(i).getFat());
                    prquery = addKeyValue(prquery,"cmcarbs[]",""+cmpls.get(i).getCarb());
                    prquery = addKeyValue(prquery,"cmgis[]",""+cmpls.get(i).getGi());
                    prquery = addKeyValue(prquery,"cmweights[]",""+cmpls.get(i).getWeight());

                    query += "&"+prquery;
                }
                
                query = addKeyValue(query,"ok","ok");

                if (!isCancelled()){
                    DoingPOST post = new DoingPOST(inetData.getServer(),query);
                    
                    if (!post.isError()){
                        log.append("Продукты экспортированы успешно\n");
                    }
                    else{
                        log.append(post.getErrorMessage()+"\n");
                    }
                }
            }
        }
            //тут надо проверить версию
            if (!hasCheckedVer){
                String r = checkVersion();
                if (r.length()>0) log.append("Доступно обновление программы\n"+r+"\n");
            }
            return null;
        }

        /*
         * Executed in event dispatch thread
         */
        @Override
        public void done() {
            Toolkit.getDefaultToolkit().beep();
        }
    }

    private User extractCoefs(String in){
        String st = in.replace("\n", "");
        //ДБ 5 параметров
        String [] par = st.split(" ");
        if (par.length!=5){
            return null;
        }
        
        User user = owner.getUser();
        user.getFactors().setK1Value( new Float(par[0]) * 
                user.getFactors().getBEValue() / 10f );
        user.getFactors().setK2(new Float(par[1]));
        user.getFactors().setK3(new Float(par[2]));

        user.setSh1(new Sugar(new Float(par[3])));
        user.setSh2(new Sugar(new Float(par[4])));
        return user;
    }

    private JComponent createBottomPane(){
        JPanel pane = new JPanel();
        log = new JTextArea();
        log.getDocument().addDocumentListener(new DocumentListener(){
            public void insertUpdate(DocumentEvent e){
                log.setCaretPosition(log.getText().length()-1);
            }
            public void removeUpdate(DocumentEvent e){}
            public void changedUpdate(DocumentEvent e){}

        });
        log.setEditable(false);
        scr = new JScrollPane(log);
        scr.setPreferredSize(new Dimension(100,100));
        prBar = new JProgressBar();
        JButton closeForm = new JButton("Закрыть");
        closeForm.addActionListener(new ActionListener(){
            public void actionPerformed(ActionEvent ev){
                closeDialog();
            }
        });
        closeForm.setFont(settings.getIn().getFont(closeForm.getFont()));

        GroupLayout layout = new GroupLayout(pane);
        pane.setLayout(layout);

        layout.setAutoCreateGaps(true);
        layout.setAutoCreateContainerGaps(true);

        layout.setHorizontalGroup(
            layout.createParallelGroup()
                .addComponent(scr)
                .addComponent(prBar)
                .addComponent(closeForm,GroupLayout.Alignment.CENTER)
             );
        layout.setVerticalGroup(
            layout.createSequentialGroup()
                .addComponent(scr)
                .addComponent(prBar)
                .addComponent(closeForm)
             );
        return pane;
    }
    private JComponent createTopPane(){
        MyFocusListener fcLst = new MyFocusListener();


        JPanel pane = new JPanel();
        JLabel lblLogin = new JLabel(settings.getIn().StrSize("Логин:"));
        lblLogin.setHorizontalAlignment(SwingConstants.TRAILING);
        fldLogin = new JTextField();
        Font fn = settings.getIn().getFont(fldLogin.getFont());
        fldLogin.setFont(fn);
        //fldLogin.setColumns(20);
        fldLogin.setText(inetData.getLogin());
        fldLogin.addFocusListener(fcLst);
        
        JLabel lblPass = new JLabel(settings.getIn().StrSize("Пароль:"));
        lblPass.setHorizontalAlignment(SwingConstants.TRAILING);
        fldPass = new JPasswordField();
        fldPass.setFont(fn);
        //fldPass.setColumns(20);
        fldPass.setText(inetData.getPass());
        btnChPass = new JButton();
        btnChPass.setMargin(new Insets(0,0,0,0));
        btnChPass.setToolTipText(settings.getIn().StrSize("Сменить пароль"));
        btnChPass.setIcon(new ImageIcon(InetBackUpDialog.class.getResource("bttns/ChPass.png")));
        btnChPass.setActionCommand(CHANGE_PASS);
        btnChPass.addActionListener(this);


        JLabel lblServer = new JLabel(settings.getIn().StrSize("Адрес сервера:"));
        lblServer.setHorizontalAlignment(SwingConstants.TRAILING);
        fldServer = new JTextField(inetData.getServer());
        fldServer.setFont(fn);
        //fldServer.setColumns(20);
        fldServer.setEditable(false);
        fldServer.addFocusListener(fcLst);
        fldServer.addMouseListener(new MouseAdapter(){
            @Override
            public void mouseClicked(MouseEvent e){
                if (SwingUtilities.isLeftMouseButton(e) && e.getClickCount()%2==0){
                    ((JTextField)e.getSource()).setEditable(true);
                    ((JTextField)e.getSource()).removeMouseListener(this);
                }
            }
        });
        
        //GroupLayout layout = new GroupLayout(pane);
        pane.setLayout(new GridBagLayout());

        pane.add(lblServer,
          new GridBagConstraints(0,0,1,1,0.5,0, GridBagConstraints.LINE_END,
          GridBagConstraints.NONE, new Insets(5,5,2,2), 0, 0));
        pane.add(fldServer,
          new GridBagConstraints(1,0,2,1,0.5,0, GridBagConstraints.LINE_START,
          GridBagConstraints.HORIZONTAL, new Insets(5,0,2,5), 0, 0));

        pane.add(lblLogin,
          new GridBagConstraints(0,1,1,1,0.5,0, GridBagConstraints.LINE_END,
          GridBagConstraints.NONE, new Insets(2,5,2,2), 0, 0));
        pane.add(fldLogin,
          new GridBagConstraints(1,1,2,1,0.5,0, GridBagConstraints.LINE_START,
          GridBagConstraints.HORIZONTAL, new Insets(2,0,2,5), 0, 0));

        pane.add(lblPass,
          new GridBagConstraints(0,2,1,1,0.5,0, GridBagConstraints.LINE_END,
          GridBagConstraints.NONE, new Insets(2,5,5,2), 0, 0));
        pane.add(fldPass,
          new GridBagConstraints(1,2,1,1,0.6,0, GridBagConstraints.CENTER,
          GridBagConstraints.HORIZONTAL, new Insets(2,0,5,2), 0, 0));
        pane.add(btnChPass,
          new GridBagConstraints(2,2,1,1,0.01,0, GridBagConstraints.LINE_END,
          GridBagConstraints.NONE, new Insets(2,0,5,5), 0, 0));

        

        return pane;
    }
    private JComponent createSyncPane(){
        JLabel lblSync = new JLabel(settings.getIn().StrSize(
                "<center>Синхронизация баз программы<br>с базами в интернет</center>"
                ));
        lblSync.setHorizontalAlignment(SwingConstants.CENTER);
        btnSync = new JButton("Синхронизировать");
        btnSync.setFont(settings.getIn().getFont(btnSync.getFont()));
        btnSync.addActionListener(this);
        btnSync.setActionCommand(BEGIN_SYNC);

        JLabel explain = new JLabel(settings.getIn().StrSize(
                "В начале программа импортирует в локальную базу,<br>" +
                "ориентируясь на выбранный приоритет баз,<br>" +
                "продукты с сервера в режиме объединения, а затем<br>" +
                "экспортирует на сервер уже объединенную локальную базу.<br>" +
                "В результате локальная база и база на сервере становятся<br>" +
                "одинаковыми."));
        rbLocalMajor = new JRadioButton(settings.getIn().StrSize(
                "Приоритет продуктов в локальной базе"));
        rbServerMajor = new JRadioButton(settings.getIn().StrSize(
                "Приоритет продуктов на сервере"));
        
        rbLocalMajor.setSelected(true);
        ButtonGroup bgr = new ButtonGroup();
        bgr.add(rbServerMajor);
        bgr.add(rbLocalMajor);

        JPanel pane = new JPanel();

        GroupLayout layout = new GroupLayout(pane);
        pane.setLayout(layout);

        layout.setAutoCreateGaps(true);
        layout.setAutoCreateContainerGaps(true);

        layout.setHorizontalGroup(
                layout.createParallelGroup(GroupLayout.Alignment.CENTER)
                    .addComponent(lblSync)
                    
                    .addComponent(explain)

                    .addComponent(rbServerMajor,GroupLayout.Alignment.LEADING)
                    .addComponent(rbLocalMajor,GroupLayout.Alignment.LEADING)
                    .addComponent(btnSync)
                );
        layout.setVerticalGroup(
                layout.createSequentialGroup()
                    .addComponent(lblSync)
                    
                    .addComponent(explain)
                    .addComponent(rbServerMajor)
                    .addComponent(rbLocalMajor)
                    .addComponent(btnSync)
                );
        return pane;
    }
    private JComponent createExportPane(){
        JLabel lblTop = new JLabel(settings.getIn().StrSize("Выберите сохраняемые на сервер элементы"));
        chExMenu = new JCheckBox(settings.getIn().StrSize("Меню, коэффициенты и рассчет"));
        chExProducts = new JCheckBox(settings.getIn().StrSize(
                "База продуктов <i>база на сервере<br>" +
                "будет заменена на сохраняемую базу</i>"));
        

        btnStExport = new JButton("Начать");
        btnStExport.setFont(settings.getIn().getFont(btnStExport.getFont()));
        btnStExport.addActionListener(this);
        btnStExport.setActionCommand(BEGIN_EXPORT);

        JPanel pane = new JPanel();

        GroupLayout layout = new GroupLayout(pane);
        pane.setLayout(layout);

        layout.setAutoCreateGaps(true);
        layout.setAutoCreateContainerGaps(true);

        layout.setHorizontalGroup(
                layout.createParallelGroup()
                    .addComponent(lblTop)
                    .addComponent(chExMenu)
                    .addComponent(chExProducts)
                    .addComponent(btnStExport,GroupLayout.Alignment.CENTER)
                    );
        layout.setVerticalGroup(
                layout.createSequentialGroup()
                    .addComponent(lblTop)
                    .addComponent(chExMenu)
                    .addComponent(chExProducts)
                    .addPreferredGap(LayoutStyle.ComponentPlacement.RELATED,
                        GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .addComponent(btnStExport)
                    );

        return pane;
    }

    private JRadioButton createRdBtn(String name,String tooltip){
        final JRadioButton rb = new JRadioButton(settings.getIn().StrSize(name));
        final JPanel tooltipPane = new JPanel();
        JLabel lbl = new JLabel(settings.getIn().StrSize(tooltip));
        
        tooltipPane.setBorder(BorderFactory.createRaisedBevelBorder());
        tooltipPane.add(lbl);
        final JWindow w = new JWindow();
        w.add(tooltipPane);
        w.setSize(lbl.getPreferredSize().width+30, lbl.getPreferredSize().height+20);
        rb.addMouseListener(new MouseAdapter(){
            @Override
            public void mouseEntered(MouseEvent ev){
                if ( ((JRadioButton)ev.getSource()).isEnabled() ){
                    int x = ((JRadioButton)ev.getSource()).getLocationOnScreen().x + 30;
                    int y = ((JRadioButton)ev.getSource()).getLocationOnScreen().y +
                            ((JRadioButton)ev.getSource()).getHeight()+1;
                    w.setLocation(x, y);
                    w.setVisible(true);
                }
            }
            @Override
            public void mouseExited(MouseEvent ev){
                if ( ((JRadioButton)ev.getSource()).isEnabled() && w.isVisible())
                w.setVisible(false);
            }
        });


        return rb;
    }
    private JComponent createImportPane(){
        JLabel lblTop = new JLabel(settings.getIn().StrSize("Выберите загружаемые с сервера элементы"));
        chImMenu = new JCheckBox(settings.getIn().StrSize("Меню, коэффициенты и рассчет"));
        chImProducts = new JCheckBox(settings.getIn().StrSize("База продуктов"));
        
        rbIm1 = createRdBtn("Объединение","В этом режиме происходит объединение<br>" +
                       "баз с сервера и локальной базы, если на<br>" +
                       "сервере есть какие то продукты или группы,<br>" +
                       "которых нет в локальной базе, то они будут<br>" +
                       "добавлены. Это предпочтительный режим");
        rbIm2 = createRdBtn("Добавление",
                    "В этом режиме происходит добавление<br>" +
                       "баз с сервера к локальной базе, если<br>" +
                       "на сервере есть какие то продукты или<br>" +
                       "группы, то они будут добавлены после<br>" +
                       "существующих продуктов."
                );
        rbIm3 = createRdBtn("Очистка и добавление",
                "В этом режиме существующая локально база<br>" +
                "продуктов удаляется <u>БЕЗВОЗВРАТНО</u> и<br>" +
                "заменяется базой продуктов, скачанной с<br>" +
                "сервера.<br><br>" +
                "<u>Используйте с осторожностью!</u>"
                );

        chImProducts.addItemListener(new ItemListener(){
            public void itemStateChanged(ItemEvent ev){
                rbIm1.setEnabled(chImProducts.isSelected());
                rbIm2.setEnabled(chImProducts.isSelected());
                rbIm3.setEnabled(chImProducts.isSelected());

            }
        });
        rbIm1.setEnabled(false);
        rbIm2.setEnabled(false);
        rbIm3.setEnabled(false);

        ButtonGroup btnGr = new ButtonGroup();
        btnGr.add(rbIm1);
        btnGr.add(rbIm2);
        btnGr.add(rbIm3);
        rbIm1.setSelected(true);

        chImDiary = new JCheckBox(settings.getIn().StrSize("Записи дневника"));

        btnStImport = new JButton("Начать");
        btnStImport.setFont(settings.getIn().getFont(btnStImport.getFont()));
        btnStImport.addActionListener(this);
        btnStImport.setActionCommand(BEGIN_IMPORT);

        JPanel pane = new JPanel();

        GroupLayout layout = new GroupLayout(pane);
        pane.setLayout(layout);

        layout.setAutoCreateGaps(true);
        layout.setAutoCreateContainerGaps(true);

        layout.setHorizontalGroup(
                layout.createParallelGroup()
                    .addComponent(lblTop)
                    .addComponent(chImMenu)
                    .addComponent(chImProducts)
                    .addGroup(layout.createSequentialGroup()
                        .addGap(15)
                        .addComponent(rbIm1)
                    )
                    .addGroup(layout.createSequentialGroup()
                        .addGap(15)
                        .addComponent(rbIm2)
                    )
                    .addGroup(layout.createSequentialGroup()
                        .addGap(15)
                        .addComponent(rbIm3)
                    )
                    .addComponent(chImDiary)
                    .addComponent(btnStImport,GroupLayout.Alignment.CENTER)
                    );
        layout.setVerticalGroup(
                layout.createSequentialGroup()
                    .addComponent(lblTop)
                    .addComponent(chImMenu)
                    .addComponent(chImProducts)
                    .addComponent(rbIm1)
                    .addComponent(rbIm2)
                    .addComponent(rbIm3)
                    .addComponent(chImDiary)
                    .addPreferredGap(LayoutStyle.ComponentPlacement.RELATED,
                        GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .addComponent(btnStImport)
                    );

        return pane;
    }
}
